% File: V2_endsc.m
% Purpose: V2 layer->enstopped cells (curvature and sign)
% Parameters:
%   - V1: structure containing V1 neuron responses
%   - cs: Vector of gain for small neurons
%   - s: Vector of size of neurons
% Returns:
%   V2: struct with simple and endstopped neuron values.
%    - max is the maximum values,
%      these are not used but left just in case is needed in the future
%    - corner: corner neuron values, not used for the paper
%    - endsc: complex endstopped neurons, like in the paper
%    - signp: values of positive sign endstopped neurons
%    - signn: values of negative sign endstopped neurons
%
% Author: Antonio J. Rodriguez-Sanchez
%
% CVR, Intelligent and Interactive Systems
% Copyright (c) 2012, York University/University of Innsbruck
%
% Permission is granted to copy and distribute this file, for noncommercial
% use, provided (a) this copyright notice is preserved, (b) no attempt
% is made to restrict redistribution of this file, and (c) this file is
% restricted by a compilation copyright.
%
%  All comments concerning this program may be sent to the
%  e-mail address 'Antonio.Rodriguez-Sanchez@uibk.ac.at'
%

function V2 = V2_endsc(V1,cs,s)

V2 = struct('max', zeros(size(V1.simple_e,3), size(V1.simple_e,4)), ...
    'endsc', zeros(size(V1.simple_e)), ...
    'corner', zeros(size(V1.simple_e,2),size(V1.simple_e,3),size(V1.simple_e,4)), ...
    'signp', zeros(size(V1.simple_e)), ...
    'signn', zeros(size(V1.simple_e)));

% Rectificar V1 --------
V1.simple_e(V1.simple_e<=0) = 0.001;
V1.complex_e(V1.complex_e<=0) = 0.001;
ndir = size(V1.simple_e,1);

for i=1:ndir 
  th = (i-1)*pi/ndir;
  
  % ---------------- Complex endstopped ------------
  for j=size(V1.simple_e,2):-1:1
    auxe = V1.complex_e(i,j,:,:);
    auxs1 = V1.complex_e(i,j,:,:); auxs2 = V1.complex_e(i,j,:,:);
    auxs1 = auxs1+V1.complex_e(mod(i-1+floor(ndir/4),ndir)+1,j,:,:);
    auxs2 = auxs2+V1.complex_e(mod(i-1+floor(3*ndir/4),ndir)+1,j,:,:);
    a = int16(round(s(j)/2*sin(th))); as = a;
    b = int16(round(s(j)/2*cos(th))); bs = b;
    if j==4
      as = int16(round(s(j)/5*sin(th)));
      bs = int16(round(s(j)/5*cos(th)));
    end
    if j==3
      as = int16(1*round(s(j)/4*sin(th)));
      bs = int16(1*round(s(j)/4*cos(th)));
    end
    if j==2
      as = int16(1*round(s(j)/4*sin(th)));
      bs = int16(1*round(s(j)/4*cos(th)));
    end
    if j==1
      as = int16(2*round(s(j)/5*sin(th)));
      bs = int16(2*round(s(j)/5*cos(th)));
    end
    % Esto funciona, ha sido comprobado
    if(b<=0)
      b = abs(b); bs = abs(bs);
      V2.endsc(i,j,a+1:size(auxe,3)-a,b+1:size(auxe,4)-b) = ...
        cs(j)*V1.simple_e(i,j,a+1:size(auxe,3)-a,b+1:size(auxe,4)-b) - ...
        auxe(1,1,1:size(auxe,3)-2*a,1:size(auxe,4)-2*b) - ...
        auxe(1,1,2*a+1:size(auxe,3),2*b+1:size(auxe,4));
      V2.signp(i,j,as+1:size(auxe,3)-as,bs+1:size(auxe,4)-bs) = ...
        cs(j)*V1.simple_e(i,j,as+1:size(auxe,3)-as,bs+1:size(auxe,4)-bs) - ...
        auxs2(1,1,1:size(auxe,3)-2*as,1:size(auxe,4)-2*bs) - ...
        auxs1(1,1,2*as+1:size(auxe,3),2*bs+1:size(auxe,4));      
      V2.signn(i,j,as+1:size(auxe,3)-as,bs+1:size(auxe,4)-bs) = ...
        cs(j)*V1.simple_e(i,j,as+1:size(auxe,3)-as,bs+1:size(auxe,4)-bs) - ...
        auxs1(1,1,1:size(auxe,3)-2*as,1:size(auxe,4)-2*bs) - ...
        auxs2(1,1,2*as+1:size(auxe,3),2*bs+1:size(auxe,4));      
    else
      V2.endsc(i,j,a+1:size(auxe,3)-a,b+1:size(auxe,4)-b) = ...
        cs(j)*V1.simple_e(i,j,a+1:size(auxe,3)-a,b+1:size(auxe,4)-b) - ...
        auxe(1,1,1:size(auxe,3)-2*a,2*b+1:size(auxe,4)) - ...
        auxe(1,1,2*a+1:size(auxe,3),1:size(auxe,4)-2*b);      
      V2.signp(i,j,as+1:size(auxe,3)-as,bs+1:size(auxe,4)-bs) = ...
        cs(j)*V1.simple_e(i,j,as+1:size(auxe,3)-as,bs+1:size(auxe,4)-bs) - ...
        auxs1(1,1,1:size(auxe,3)-2*as,2*bs+1:size(auxe,4)) - ...
        auxs2(1,1,2*as+1:size(auxe,3),1:size(auxe,4)-2*bs);
      V2.signn(i,j,as+1:size(auxe,3)-as,bs+1:size(auxe,4)-bs) = ...
        cs(j)*V1.simple_e(i,j,as+1:size(auxe,3)-as,bs+1:size(auxe,4)-bs) - ...
        auxs2(1,1,1:size(auxe,3)-2*as,2*bs+1:size(auxe,4)) - ...
        auxs1(1,1,2*as+1:size(auxe,3),1:size(auxe,4)-2*bs);
    end
  end
end

% Esto es para las esquinas
% for j=1:ndir
%   aux = zeros(1,size(V1.simple_e,2),size(V1.simple_e,3),size(V1.simple_e,4));
%   for i=1:ndir
%     aux = max(aux,V1.simple_e(i,:,:,:)+V1.simple_e(mod(i+j-1,ndir)+1,:,:,:));
%   end
%   V2.corner(j,:,:,:) = aux;
% end;

V2.corner = sum(V1.simple_e,[]);

end
